"""[DO NOT EDIT MANUALLY] Auto-generated model classes.

Auto-generated by Hera via `make events-models`.
OpenAPI spec URL: https://raw.githubusercontent.com/argoproj/argo-workflows/v3.6.2/api/openapi-spec/swagger.json
"""

from hera.events.models.eventsource import (
    CreateEventSourceRequest,
    EventSourceDeletedResponse,
    EventSourceWatchEvent,
    LogEntry as EventsourceLogEntry,
    UpdateEventSourceRequest,
)
from hera.events.models.io.argoproj.events.v1alpha1 import (
    Amount,
    AMQPConsumeConfig,
    AMQPEventSource,
    AMQPExchangeDeclareConfig,
    AMQPQueueBindConfig,
    AMQPQueueDeclareConfig,
    ArgoWorkflowTrigger,
    ArtifactLocation,
    AWSLambdaTrigger,
    AzureEventHubsTrigger,
    AzureEventsHubEventSource,
    AzureQueueStorageEventSource,
    AzureServiceBusEventSource,
    AzureServiceBusTrigger,
    Backoff,
    BasicAuth,
    BitbucketAuth,
    BitbucketBasicAuth,
    BitbucketEventSource,
    BitbucketRepository,
    BitbucketServerEventSource,
    BitbucketServerRepository,
    CalendarEventSource,
    CatchupConfiguration,
    Condition,
    ConditionsResetByTime,
    ConditionsResetCriteria,
    ConfigMapPersistence,
    CustomTrigger,
    DataFilter,
    EmailTrigger,
    EmitterEventSource,
    EventContext,
    EventDependency,
    EventDependencyFilter,
    EventDependencyTransformer,
    EventPersistence,
    EventSource,
    EventSourceFilter,
    EventSourceList,
    EventSourceSpec,
    EventSourceStatus,
    ExprFilter,
    FileArtifact,
    FileEventSource,
    GenericEventSource,
    GerritEventSource,
    GitArtifact,
    GitCreds,
    GithubAppCreds,
    GithubEventSource,
    GitlabEventSource,
    GitRemoteConfig,
    HDFSEventSource,
    HTTPTrigger,
    Int64OrString,
    K8SResourcePolicy,
    KafkaConsumerGroup,
    KafkaEventSource,
    KafkaTrigger,
    LogTrigger,
    Metadata,
    MQTTEventSource,
    NATSAuth,
    NATSEventsSource,
    NATSTrigger,
    NSQEventSource,
    OpenWhiskTrigger,
    OwnedRepositories,
    PayloadField,
    PubSubEventSource,
    PulsarEventSource,
    PulsarTrigger,
    RateLimit,
    RedisEventSource,
    RedisStreamEventSource,
    Resource,
    ResourceEventSource,
    ResourceFilter,
    S3Artifact,
    S3Bucket,
    S3Filter,
    SASLConfig,
    SchemaRegistryConfig,
    SecureHeader,
    Selector,
    Sensor,
    SensorList,
    SensorSpec,
    SensorStatus,
    Service,
    SFTPEventSource,
    SlackEventSource,
    SlackSender,
    SlackThread,
    SlackTrigger,
    SNSEventSource,
    SQSEventSource,
    StandardK8STrigger,
    Status,
    StatusPolicy,
    StorageGridEventSource,
    StorageGridFilter,
    StripeEventSource,
    Template,
    TimeFilter,
    TLSConfig,
    Trigger,
    TriggerParameter,
    TriggerParameterSource,
    TriggerPolicy,
    TriggerTemplate,
    URLArtifact,
    ValueFromSource,
    WatchPathConfig,
    WebhookContext,
    WebhookEventSource,
)
from hera.events.models.io.argoproj.workflow.v1alpha1 import (
    Event,
    EventResponse,
    GetUserInfoResponse,
    InfoResponse,
    Item,
    Version,
)
from hera.events.models.io.k8s.api.core.v1 import ImagePullPolicy
from hera.events.models.sensor import (
    CreateSensorRequest,
    DeleteSensorResponse,
    LogEntry as SensorLogEntry,
    SensorWatchEvent,
    UpdateSensorRequest,
)

__all__ = [
    "AMQPConsumeConfig",
    "AMQPEventSource",
    "AMQPExchangeDeclareConfig",
    "AMQPQueueBindConfig",
    "AMQPQueueDeclareConfig",
    "AWSLambdaTrigger",
    "Amount",
    "ArgoWorkflowTrigger",
    "ArtifactLocation",
    "AzureEventHubsTrigger",
    "AzureEventsHubEventSource",
    "AzureQueueStorageEventSource",
    "AzureServiceBusEventSource",
    "AzureServiceBusTrigger",
    "Backoff",
    "BasicAuth",
    "BitbucketAuth",
    "BitbucketBasicAuth",
    "BitbucketEventSource",
    "BitbucketRepository",
    "BitbucketServerEventSource",
    "BitbucketServerRepository",
    "CalendarEventSource",
    "CatchupConfiguration",
    "Condition",
    "ConditionsResetByTime",
    "ConditionsResetCriteria",
    "ConfigMapPersistence",
    "CreateEventSourceRequest",
    "CreateSensorRequest",
    "CustomTrigger",
    "DataFilter",
    "DeleteSensorResponse",
    "EmailTrigger",
    "EmitterEventSource",
    "Event",
    "EventContext",
    "EventDependency",
    "EventDependencyFilter",
    "EventDependencyTransformer",
    "EventPersistence",
    "EventResponse",
    "EventSource",
    "EventSourceDeletedResponse",
    "EventSourceFilter",
    "EventSourceList",
    "EventSourceSpec",
    "EventSourceStatus",
    "EventSourceWatchEvent",
    "EventsourceLogEntry",
    "ExprFilter",
    "FileArtifact",
    "FileEventSource",
    "GenericEventSource",
    "GerritEventSource",
    "GetUserInfoResponse",
    "GitArtifact",
    "GitCreds",
    "GitRemoteConfig",
    "GithubAppCreds",
    "GithubEventSource",
    "GitlabEventSource",
    "HDFSEventSource",
    "HTTPTrigger",
    "ImagePullPolicy",
    "InfoResponse",
    "Int64OrString",
    "Item",
    "K8SResourcePolicy",
    "KafkaConsumerGroup",
    "KafkaEventSource",
    "KafkaTrigger",
    "LogTrigger",
    "MQTTEventSource",
    "Metadata",
    "NATSAuth",
    "NATSEventsSource",
    "NATSTrigger",
    "NSQEventSource",
    "OpenWhiskTrigger",
    "OwnedRepositories",
    "PayloadField",
    "PubSubEventSource",
    "PulsarEventSource",
    "PulsarTrigger",
    "RateLimit",
    "RedisEventSource",
    "RedisStreamEventSource",
    "Resource",
    "ResourceEventSource",
    "ResourceFilter",
    "S3Artifact",
    "S3Bucket",
    "S3Filter",
    "SASLConfig",
    "SFTPEventSource",
    "SNSEventSource",
    "SQSEventSource",
    "SchemaRegistryConfig",
    "SecureHeader",
    "Selector",
    "Sensor",
    "SensorList",
    "SensorLogEntry",
    "SensorSpec",
    "SensorStatus",
    "SensorWatchEvent",
    "Service",
    "SlackEventSource",
    "SlackSender",
    "SlackThread",
    "SlackTrigger",
    "StandardK8STrigger",
    "Status",
    "StatusPolicy",
    "StorageGridEventSource",
    "StorageGridFilter",
    "StripeEventSource",
    "TLSConfig",
    "Template",
    "TimeFilter",
    "Trigger",
    "TriggerParameter",
    "TriggerParameterSource",
    "TriggerPolicy",
    "TriggerTemplate",
    "URLArtifact",
    "UpdateEventSourceRequest",
    "UpdateSensorRequest",
    "ValueFromSource",
    "Version",
    "WatchPathConfig",
    "WebhookContext",
    "WebhookEventSource",
]
