# generated by datamodel-codegen:
#   filename:  argo-workflows-3.6.2.json

from __future__ import annotations

from typing import Union

from hera.shared._pydantic import BaseModel


class IntOrString(BaseModel):
    __root__: Union[str, int]
