# generated by datamodel-codegen:
#   filename:  argo-workflows-3.6.2.json

from __future__ import annotations

from typing import Optional

from hera.shared._pydantic import BaseModel


class Any(BaseModel):
    type_url: Optional[str] = None
    value: Optional[str] = None
