"""[DO NOT EDIT MANUALLY] Auto-generated model classes.

Auto-generated by Hera via `make workflows-models`.
OpenAPI spec URL: https://raw.githubusercontent.com/argoproj/argo-workflows/v3.6.2/api/openapi-spec/swagger.json
"""

from hera.workflows.models.google.protobuf import Any
from hera.workflows.models.grpc.gateway.runtime import Error, StreamError
from hera.workflows.models.io.argoproj.workflow.v1alpha1 import (
    HTTP,
    Amount,
    ArchivedWorkflowDeletedResponse,
    ArchiveStrategy,
    Arguments,
    ArtGCStatus,
    Artifact,
    ArtifactGC,
    ArtifactLocation,
    ArtifactoryArtifact,
    ArtifactoryArtifactRepository,
    ArtifactPaths,
    ArtifactRepository,
    ArtifactRepositoryRef,
    ArtifactRepositoryRefStatus,
    AzureArtifact,
    AzureArtifactRepository,
    Backoff,
    BasicAuth,
    Cache,
    ClientCertAuth,
    ClusterWorkflowTemplate,
    ClusterWorkflowTemplateCreateRequest,
    ClusterWorkflowTemplateDeleteResponse,
    ClusterWorkflowTemplateLintRequest,
    ClusterWorkflowTemplateList,
    ClusterWorkflowTemplateUpdateRequest,
    CollectEventRequest,
    CollectEventResponse,
    Column,
    Condition,
    ContainerNode,
    ContainerSetRetryStrategy,
    ContainerSetTemplate,
    ContinueOn,
    Counter,
    CreateCronWorkflowRequest,
    CreateS3BucketOptions,
    CronWorkflow,
    CronWorkflowDeletedResponse,
    CronWorkflowList,
    CronWorkflowResumeRequest,
    CronWorkflowSpec,
    CronWorkflowStatus,
    CronWorkflowSuspendRequest,
    DAGTask,
    DAGTemplate,
    Data,
    DataSource,
    Event,
    EventResponse,
    ExecutorConfig,
    Gauge,
    GCSArtifact,
    GCSArtifactRepository,
    GetUserInfoResponse,
    GitArtifact,
    HDFSArtifact,
    HDFSArtifactRepository,
    Header,
    Histogram,
    HTTPArtifact,
    HTTPAuth,
    HTTPBodySource,
    HTTPHeader as V1alpha1HTTPHeader,
    HTTPHeaderSource,
    InfoResponse,
    Inputs,
    Item,
    LabelKeys,
    LabelValueFrom,
    LabelValues,
    LifecycleHook,
    Link,
    LintCronWorkflowRequest,
    LogEntry as V1alpha1LogEntry,
    ManifestFrom,
    MemoizationStatus,
    Memoize,
    Metadata,
    MetricLabel,
    Metrics,
    Mutex,
    MutexHolding,
    MutexStatus,
    NodeFlag,
    NodeStatus,
    NodeSynchronizationStatus,
    NoneStrategy,
    OAuth2Auth,
    OAuth2EndpointParam,
    OSSArtifact,
    OSSArtifactRepository,
    OSSLifecycleRule,
    Outputs,
    ParallelSteps,
    Parameter,
    Plugin,
    PodGC,
    Prometheus,
    RawArtifact,
    ResourceTemplate,
    ResubmitArchivedWorkflowRequest,
    RetryAffinity,
    RetryArchivedWorkflowRequest,
    RetryNodeAntiAffinity,
    RetryStrategy,
    S3Artifact,
    S3ArtifactRepository,
    S3EncryptionOptions,
    ScriptTemplate,
    SemaphoreHolding,
    SemaphoreRef,
    SemaphoreStatus,
    Sequence,
    StopStrategy,
    Submit,
    SubmitOpts,
    SuppliedValueFrom,
    SuspendTemplate,
    Synchronization,
    SynchronizationStatus,
    TarStrategy,
    Template,
    TemplateRef,
    TransformationStep,
    TTLStrategy,
    UpdateCronWorkflowRequest,
    UserContainer,
    ValueFrom,
    Version,
    VolumeClaimGC,
    Workflow,
    WorkflowCreateRequest,
    WorkflowDeleteResponse,
    WorkflowEventBinding,
    WorkflowEventBindingList,
    WorkflowEventBindingSpec,
    WorkflowLevelArtifactGC,
    WorkflowLintRequest,
    WorkflowList,
    WorkflowMetadata,
    WorkflowResubmitRequest,
    WorkflowResumeRequest,
    WorkflowRetryRequest,
    WorkflowSetRequest,
    WorkflowSpec,
    WorkflowStatus,
    WorkflowStep,
    WorkflowStopRequest,
    WorkflowSubmitRequest,
    WorkflowSuspendRequest,
    WorkflowTemplate,
    WorkflowTemplateCreateRequest,
    WorkflowTemplateDeleteResponse,
    WorkflowTemplateLintRequest,
    WorkflowTemplateList,
    WorkflowTemplateRef,
    WorkflowTemplateUpdateRequest,
    WorkflowTerminateRequest,
    WorkflowWatchEvent,
    ZipStrategy,
)
from hera.workflows.models.io.k8s.api.core.v1 import (
    Affinity,
    AppArmorProfile,
    AWSElasticBlockStoreVolumeSource,
    AzureDiskVolumeSource,
    AzureFileVolumeSource,
    Capabilities,
    CephFSVolumeSource,
    CinderVolumeSource,
    ClusterTrustBundleProjection,
    ConfigMapEnvSource,
    ConfigMapKeySelector,
    ConfigMapProjection,
    ConfigMapVolumeSource,
    Container,
    ContainerPort,
    ContainerResizePolicy,
    CSIVolumeSource,
    DownwardAPIProjection,
    DownwardAPIVolumeFile,
    DownwardAPIVolumeSource,
    EmptyDirVolumeSource,
    EnvFromSource,
    EnvVar,
    EnvVarSource,
    EphemeralVolumeSource,
    ExecAction,
    FCVolumeSource,
    FlexVolumeSource,
    FlockerVolumeSource,
    GCEPersistentDiskVolumeSource,
    GitRepoVolumeSource,
    GlusterfsVolumeSource,
    GRPCAction,
    HostAlias,
    HostPathVolumeSource,
    HTTPGetAction,
    HTTPHeader as V1HTTPHeader,
    ImagePullPolicy,
    ISCSIVolumeSource,
    KeyToPath,
    Lifecycle,
    LifecycleHandler,
    LocalObjectReference,
    ModifyVolumeStatus,
    NFSVolumeSource,
    NodeAffinity,
    NodeSelector,
    NodeSelectorRequirement,
    NodeSelectorTerm,
    ObjectFieldSelector,
    ObjectReference,
    PersistentVolumeClaim,
    PersistentVolumeClaimCondition,
    PersistentVolumeClaimSpec,
    PersistentVolumeClaimStatus,
    PersistentVolumeClaimTemplate,
    PersistentVolumeClaimVolumeSource,
    PhotonPersistentDiskVolumeSource,
    PodAffinity,
    PodAffinityTerm,
    PodAntiAffinity,
    PodDNSConfig,
    PodDNSConfigOption,
    PodSecurityContext,
    PortworxVolumeSource,
    PreferredSchedulingTerm,
    Probe,
    ProjectedVolumeSource,
    QuobyteVolumeSource,
    RBDVolumeSource,
    ResourceClaim,
    ResourceFieldSelector,
    ResourceRequirements,
    ScaleIOVolumeSource,
    SeccompProfile,
    SecretEnvSource,
    SecretKeySelector,
    SecretProjection,
    SecretVolumeSource,
    SecurityContext,
    SELinuxOptions,
    ServiceAccountTokenProjection,
    ServicePort,
    SleepAction,
    StorageOSVolumeSource,
    Sysctl,
    TCPSocketAction,
    Toleration,
    TypedLocalObjectReference,
    TypedObjectReference,
    Volume,
    VolumeDevice,
    VolumeMount,
    VolumeProjection,
    VolumeResourceRequirements,
    VsphereVirtualDiskVolumeSource,
    WeightedPodAffinityTerm,
    WindowsSecurityContextOptions,
)
from hera.workflows.models.io.k8s.api.policy.v1 import PodDisruptionBudgetSpec
from hera.workflows.models.io.k8s.apimachinery.pkg.api.resource import Quantity
from hera.workflows.models.io.k8s.apimachinery.pkg.apis.meta.v1 import (
    CreateOptions,
    FieldsV1,
    GroupVersionResource,
    LabelSelector,
    LabelSelectorRequirement,
    ListMeta,
    ManagedFieldsEntry,
    MicroTime,
    ObjectMeta,
    OwnerReference,
    Time,
)
from hera.workflows.models.io.k8s.apimachinery.pkg.util.intstr import IntOrString

__all__ = [
    "AWSElasticBlockStoreVolumeSource",
    "Affinity",
    "Amount",
    "Any",
    "AppArmorProfile",
    "ArchiveStrategy",
    "ArchivedWorkflowDeletedResponse",
    "Arguments",
    "ArtGCStatus",
    "Artifact",
    "ArtifactGC",
    "ArtifactLocation",
    "ArtifactPaths",
    "ArtifactRepository",
    "ArtifactRepositoryRef",
    "ArtifactRepositoryRefStatus",
    "ArtifactoryArtifact",
    "ArtifactoryArtifactRepository",
    "AzureArtifact",
    "AzureArtifactRepository",
    "AzureDiskVolumeSource",
    "AzureFileVolumeSource",
    "Backoff",
    "BasicAuth",
    "CSIVolumeSource",
    "Cache",
    "Capabilities",
    "CephFSVolumeSource",
    "CinderVolumeSource",
    "ClientCertAuth",
    "ClusterTrustBundleProjection",
    "ClusterWorkflowTemplate",
    "ClusterWorkflowTemplateCreateRequest",
    "ClusterWorkflowTemplateDeleteResponse",
    "ClusterWorkflowTemplateLintRequest",
    "ClusterWorkflowTemplateList",
    "ClusterWorkflowTemplateUpdateRequest",
    "CollectEventRequest",
    "CollectEventResponse",
    "Column",
    "Condition",
    "ConfigMapEnvSource",
    "ConfigMapKeySelector",
    "ConfigMapProjection",
    "ConfigMapVolumeSource",
    "Container",
    "ContainerNode",
    "ContainerPort",
    "ContainerResizePolicy",
    "ContainerSetRetryStrategy",
    "ContainerSetTemplate",
    "ContinueOn",
    "Counter",
    "CreateCronWorkflowRequest",
    "CreateOptions",
    "CreateS3BucketOptions",
    "CronWorkflow",
    "CronWorkflowDeletedResponse",
    "CronWorkflowList",
    "CronWorkflowResumeRequest",
    "CronWorkflowSpec",
    "CronWorkflowStatus",
    "CronWorkflowSuspendRequest",
    "DAGTask",
    "DAGTemplate",
    "Data",
    "DataSource",
    "DownwardAPIProjection",
    "DownwardAPIVolumeFile",
    "DownwardAPIVolumeSource",
    "EmptyDirVolumeSource",
    "EnvFromSource",
    "EnvVar",
    "EnvVarSource",
    "EphemeralVolumeSource",
    "Error",
    "Event",
    "EventResponse",
    "ExecAction",
    "ExecutorConfig",
    "FCVolumeSource",
    "FieldsV1",
    "FlexVolumeSource",
    "FlockerVolumeSource",
    "GCEPersistentDiskVolumeSource",
    "GCSArtifact",
    "GCSArtifactRepository",
    "GRPCAction",
    "Gauge",
    "GetUserInfoResponse",
    "GitArtifact",
    "GitRepoVolumeSource",
    "GlusterfsVolumeSource",
    "GroupVersionResource",
    "HDFSArtifact",
    "HDFSArtifactRepository",
    "HTTP",
    "HTTPArtifact",
    "HTTPAuth",
    "HTTPBodySource",
    "HTTPGetAction",
    "HTTPHeaderSource",
    "Header",
    "Histogram",
    "HostAlias",
    "HostPathVolumeSource",
    "ISCSIVolumeSource",
    "ImagePullPolicy",
    "InfoResponse",
    "Inputs",
    "IntOrString",
    "Item",
    "KeyToPath",
    "LabelKeys",
    "LabelSelector",
    "LabelSelectorRequirement",
    "LabelValueFrom",
    "LabelValues",
    "Lifecycle",
    "LifecycleHandler",
    "LifecycleHook",
    "Link",
    "LintCronWorkflowRequest",
    "ListMeta",
    "LocalObjectReference",
    "ManagedFieldsEntry",
    "ManifestFrom",
    "MemoizationStatus",
    "Memoize",
    "Metadata",
    "MetricLabel",
    "Metrics",
    "MicroTime",
    "ModifyVolumeStatus",
    "Mutex",
    "MutexHolding",
    "MutexStatus",
    "NFSVolumeSource",
    "NodeAffinity",
    "NodeFlag",
    "NodeSelector",
    "NodeSelectorRequirement",
    "NodeSelectorTerm",
    "NodeStatus",
    "NodeSynchronizationStatus",
    "NoneStrategy",
    "OAuth2Auth",
    "OAuth2EndpointParam",
    "OSSArtifact",
    "OSSArtifactRepository",
    "OSSLifecycleRule",
    "ObjectFieldSelector",
    "ObjectMeta",
    "ObjectReference",
    "Outputs",
    "OwnerReference",
    "ParallelSteps",
    "Parameter",
    "PersistentVolumeClaim",
    "PersistentVolumeClaimCondition",
    "PersistentVolumeClaimSpec",
    "PersistentVolumeClaimStatus",
    "PersistentVolumeClaimTemplate",
    "PersistentVolumeClaimVolumeSource",
    "PhotonPersistentDiskVolumeSource",
    "Plugin",
    "PodAffinity",
    "PodAffinityTerm",
    "PodAntiAffinity",
    "PodDNSConfig",
    "PodDNSConfigOption",
    "PodDisruptionBudgetSpec",
    "PodGC",
    "PodSecurityContext",
    "PortworxVolumeSource",
    "PreferredSchedulingTerm",
    "Probe",
    "ProjectedVolumeSource",
    "Prometheus",
    "Quantity",
    "QuobyteVolumeSource",
    "RBDVolumeSource",
    "RawArtifact",
    "ResourceClaim",
    "ResourceFieldSelector",
    "ResourceRequirements",
    "ResourceTemplate",
    "ResubmitArchivedWorkflowRequest",
    "RetryAffinity",
    "RetryArchivedWorkflowRequest",
    "RetryNodeAntiAffinity",
    "RetryStrategy",
    "S3Artifact",
    "S3ArtifactRepository",
    "S3EncryptionOptions",
    "SELinuxOptions",
    "ScaleIOVolumeSource",
    "ScriptTemplate",
    "SeccompProfile",
    "SecretEnvSource",
    "SecretKeySelector",
    "SecretProjection",
    "SecretVolumeSource",
    "SecurityContext",
    "SemaphoreHolding",
    "SemaphoreRef",
    "SemaphoreStatus",
    "Sequence",
    "ServiceAccountTokenProjection",
    "ServicePort",
    "SleepAction",
    "StopStrategy",
    "StorageOSVolumeSource",
    "StreamError",
    "Submit",
    "SubmitOpts",
    "SuppliedValueFrom",
    "SuspendTemplate",
    "Synchronization",
    "SynchronizationStatus",
    "Sysctl",
    "TCPSocketAction",
    "TTLStrategy",
    "TarStrategy",
    "Template",
    "TemplateRef",
    "Time",
    "Toleration",
    "TransformationStep",
    "TypedLocalObjectReference",
    "TypedObjectReference",
    "UpdateCronWorkflowRequest",
    "UserContainer",
    "V1HTTPHeader",
    "V1alpha1HTTPHeader",
    "V1alpha1LogEntry",
    "ValueFrom",
    "Version",
    "Volume",
    "VolumeClaimGC",
    "VolumeDevice",
    "VolumeMount",
    "VolumeProjection",
    "VolumeResourceRequirements",
    "VsphereVirtualDiskVolumeSource",
    "WeightedPodAffinityTerm",
    "WindowsSecurityContextOptions",
    "Workflow",
    "WorkflowCreateRequest",
    "WorkflowDeleteResponse",
    "WorkflowEventBinding",
    "WorkflowEventBindingList",
    "WorkflowEventBindingSpec",
    "WorkflowLevelArtifactGC",
    "WorkflowLintRequest",
    "WorkflowList",
    "WorkflowMetadata",
    "WorkflowResubmitRequest",
    "WorkflowResumeRequest",
    "WorkflowRetryRequest",
    "WorkflowSetRequest",
    "WorkflowSpec",
    "WorkflowStatus",
    "WorkflowStep",
    "WorkflowStopRequest",
    "WorkflowSubmitRequest",
    "WorkflowSuspendRequest",
    "WorkflowTemplate",
    "WorkflowTemplateCreateRequest",
    "WorkflowTemplateDeleteResponse",
    "WorkflowTemplateLintRequest",
    "WorkflowTemplateList",
    "WorkflowTemplateRef",
    "WorkflowTemplateUpdateRequest",
    "WorkflowTerminateRequest",
    "WorkflowWatchEvent",
    "ZipStrategy",
]
